% InsertMutants is used to insert various mutants in a .m file.
%
% Inputs:
% scriptfile -- The original .m files in which the mutants are to added.
%               The mutants are inserted only between the comment lines
%               saying 'START HERE' and 'STOP HERE'
% MutantList -- A cell array of mutants
%               The script replaces each occurance of an element with all other
%               elements.
% MutantName -- Name of the output mutant file will start with the name
%               specified in the varible along with a Mutant Number.
% Outputs:
% Mutants files -- Each mutant is stored as a new .m file and stored in the
%                  'MatlabMutants' folder.
% ListMutants   -- Text file containing the list of Information reagarding
%                   all the mutants generated.

% (c) Chethan C U <chethan.cu@gmail.com>

scriptfile = 'model0001_m.m';        % Your file

MutantName = 'Mutant';
if ~exist('MatlabMutants','dir')
mkdir('MatlabMutants');
end
dirctPath = [pwd '\' 'MatlabMutants\'];   % This variable can be updated to any desired user path
%dirctPath = 'C:\matlab_mutants\';  % Your path

% Arithmetic Mutants
% MutantList={'+','-','*','+1+','-1+'};

% Relational Operator Mutants
% MutantList={'>','<','>=','<=','==','~='};

% Variable Mutants
MutantList={'inp1','inp2','inp3','inp4','inp5','inp6','inp7','inp8','inp9','inp10','osat0','osat1','df02','ofil2','osat3','out2','osat5','out5','osat4','ofil3','oadd1','ul','ll','out7','delayo','inro2','osat6','inro1','dtx','out6'};

% Logical Mutants
%MutantList={'and','or','xor','~and','~or','~xor'};

% Data Mutants
% MutantList={'60','60.6','59.9','20','20.2','19.8','30','29.8','3.5','3.45','3.6','10','9.8','10.1','100','101','99','40','70','0.6','0.4','0.65','250','80','79.8','80.2','249.1','250.1','10'};
% MutantList={'3.462870334744132e-003','6.925740669488265e-003','3.462870334744132e-003','1.000000000000000e+000','-1.974713551955749e+000','9.762939549585143e-001','3.072196620583717e-003','9.090909090909092e+000','-8.181818181818182e-001'};

% Read the Original .m file as a cell array
scriptdata = textread(scriptfile,'%s','delimiter','\n');

% Find line Number containing 'START HERE'. Mutants are inserted after this line.
StartHere=strfind(upper(scriptdata),'START HERE');
for q=1:length(StartHere)
    if(~isempty(StartHere{q}))
        StrtLineNo=q;
    end
end
% Find line Number containing 'END HERE'. Mutants are inserted until this
% line.
EndHere=strfind(upper(scriptdata),'END HERE');
for q=1:length(EndHere)
    if(~isempty(EndHere{q}))
        EndLineNo=q;
    end
end
% Counter to keep track of the no. of mutants.
% The same counter is used to name the mutant files.
MutantCnt=0;


for ii=1:length(MutantList)
    % Find element of MutantList in the Original .m file between 'START HERE' and 'END HERE'
    SrchRes=strfind(scriptdata(StrtLineNo:EndLineNo),MutantList{ii});
    
    % Check if element of MutantList is found
    if ~isempty(SrchRes)
        lineNos=[];
        % Find all Line Numbers containing element of MutantList
        for kk= 1: length(SrchRes)
            if ~isempty(SrchRes{kk})
                lineNos=[lineNos;kk+StrtLineNo-1];
            end
        end
        
        for kk= 1:length(lineNos)
            % Replace the element of MutantList with other elements of
            % MutantList
            for jj= 1:length(MutantList)
                % Skip for same element
                if ii~=jj
                    mutant = scriptdata;
                    NoBlnks=cell2mat(strtrim(mutant(lineNos(kk))));
                    % Skip if the line is a comment
                    if NoBlnks(1)=='%'
                        break;
                    end
                    % find if the same element is repeated in the same line
                    RepsInsameLine=strfind(mutant(lineNos(kk)),MutantList{ii});
                    RepsInsameLine=cell2mat(RepsInsameLine);
                    % for each occurance of element in a line.
                    for lll=1:length(RepsInsameLine)
                        mutant = scriptdata;
                        temp=mutant{lineNos(kk)};
                        temp(RepsInsameLine(lll):RepsInsameLine(lll)+length(MutantList{ii})-1)=[];
                        temp=[temp(1:RepsInsameLine(lll)-1) '$' temp(RepsInsameLine(lll):end)];
                        mutant(lineNos(kk))={strrep(temp,'$',MutantList{jj})};
                        % Mutant Inserted. Increment count
                        MutantCnt=MutantCnt+1;
                        % ListMutants containes a list of all mutants inserted
                        diary ListMutants
                        disp(['% Changed line: ' num2str(lineNos(kk)+1) ' from ' MutantList{ii} ' to ' MutantList{jj}]);
                        diary off
                        % Use diary to store the mutant file
                        eval(['diary ' dirctPath MutantName num2str(MutantCnt) '.m']);
                        disp(['% Changed line: ' num2str(lineNos(kk)+1) ' from ' MutantList{ii} ' to ' MutantList{jj}]);
                        for cc=1:length(mutant)
                            disp(mutant{cc})
                        end
                        diary off
                    end
                end
            end
        end
    end
end





