% This program generates mutants from the Original Model for each of the
% blocks. This has been used with the benchmark mdl file to generate the
% mutants. It is not a very general purpose script but works well with the set of files.

% (c) Prasad K <k14prasad@gmail.com>

clear all;
clc;
bdclose all;
modelname = 'Original';

% Make the directory for each type of mutant
dos('mkdir Logic');
dos('mkdir Backlash');
dos('mkdir DataType');
dos('mkdir Filter');
dos('mkdir Integrator');
dos('mkdir Saturation');
dos('mkdir Sum');
dos('mkdir Switch');
dos('mkdir Delay');
dos('mkdir Lines');
dos('mkdir Lookup');
dos('mkdir Relational');

% Load model and make a list of blocks
load_system(modelname);
model = find_system(modelname,'LookUnderMasks','all','FollowLinks','on');
model1 = model(2);
model = model(2:end);
blks = get_param(model,'BlockType');

logic ={'XOR','AND','OR','NXOR','NAND','NOR'};
relation = {'>=','<=','==','~=','<','>'};
datatype = {'double','single','int8','uint8','int16','uint16','int32','uint32'};
sumblk = {'++','+-','-+','--'};
swit = {'u2 > Threshold','u2 >= Threshold','u2 ~= 0'};
look = {'Interpolation-Use End Values','Use Input Nearest','Use Input Below','Use Input Above','Interpolation-Extrapolation'};

runs = 1;

save_system(modelname,'Duplicate');
open_system('Duplicate');

% Remove all links so that we do not spoil any libraries
for i = 1:size(blks,1)
    tmp = regexprep(model{i},'Original','Duplicate','once');
    set_param(tmp, 'LinkStatus', 'none');
end
save_system('Duplicate');

open_system(modelname);
for i = 1:size(blks,1)
    set_param(model{i}, 'LinkStatus', 'none');
end
save_system(modelname);


open_system('Duplicate');

change = 0;
k1 = 1;

% Loop forall the blocks
for m = 1:size(blks,1)
    
    % Logical blocks
    if strcmp(blks{m},'Logic')
        for k = 1:size(logic,2)
            rep = logic{k};
            open_system('Duplicate');
            save_system('Duplicate',['Duplicate' num2str(runs)]);
            open_system(['Duplicate' num2str(runs)]);
            tmp = regexprep(model{m},'Original',['Duplicate' num2str(runs)],'once');
            val = get_param(tmp,'Operator');
            if ~strcmp(val,'NOT') && ~strcmp(val,rep)
                set_param(tmp,'Operator',rep);
                set_param(tmp,'BackgroundColor','red');
                runs = runs + 1;
                save_system(['Duplicate' num2str(runs  - 1 )]);
                %close_system(['Duplicate' num2str(runs  - 1 )],0);
                CreateHarness;
                
                load_system(modelname);
                eval(['diary Duplicate' num2str(runs - 1) '_changes.txt']);
                disp(['Changed Logic in ' model{m} ' Operator from ' get_param(model{m},'Operator') ' to ' rep]);
                diary off;
                movefile(['Duplicate' num2str(runs  - 1 ) '.mdl'],[pwd '\Logic'],'f');
                movefile(['Duplicate' num2str(runs  - 1 ) '_changes.txt'],[pwd '\Logic'],'f');
            else
                close_system(['Duplicate' num2str(runs)],0);
            end
        end
    end
    
    if strcmp(blks{m},'Sum')
        for k = 1:size(sumblk,2)
            rep = sumblk{k};
            open_system('Duplicate');
            save_system('Duplicate',['Duplicate' num2str(runs)]);
            open_system(['Duplicate' num2str(runs)]);
            tmp = regexprep(model{m},'Original',['Duplicate' num2str(runs)],'once');
            val = get_param(tmp,'Inputs');
            if ~strcmp(val,rep)
                set_param(tmp,'Inputs',rep);
                set_param(tmp,'BackgroundColor','red');
                runs = runs + 1;
                save_system(['Duplicate' num2str(runs  - 1 )]);
                %close_system(['Duplicate' num2str(runs  - 1 )],0);
                
                CreateHarness;
                open_system(modelname);
                eval(['diary Duplicate' num2str(runs - 1) '_changes.txt']);
                disp(['Changed Sum in ' model{m} ' operation from ' get_param(model{m},'Inputs') ' to ' rep]);
                diary off;
                movefile(['Duplicate' num2str(runs  - 1 ) '.mdl'],[pwd '\Sum'],'f');
                movefile(['Duplicate' num2str(runs  - 1 ) '_changes.txt'],[pwd '\Sum'],'f');
            else
                close_system(['Duplicate' num2str(runs)],0);
            end
        end
    end
    
    if strcmp(blks{m},'DataTypeConversion')
        for k = 1:size(datatype,2)
            rep = datatype{k};
            open_system('Duplicate');
            save_system('Duplicate',['Duplicate' num2str(runs)]);
            open_system(['Duplicate' num2str(runs)]);
            tmp = regexprep(model{m},'Original',['Duplicate' num2str(runs)],'once');
            val = get_param(tmp,'OutDataTypeMode');
            if ~strcmp(val,rep)
                set_param(tmp,'OutDataTypeMode',rep);
                set_param(tmp,'BackgroundColor','red');
                runs = runs + 1;
                save_system(['Duplicate' num2str(runs  - 1 )]);
                %close_system(['Duplicate' num2str(runs  - 1 )],0);
                CreateHarness;
                open_system(modelname);
                eval(['diary Duplicate' num2str(runs - 1) '_changes.txt']);
                disp(['Changed DataTypeConversion in ' model{m} ' Datatype from ' get_param(model{m},'OutDataTypeMode') ' to ' rep]);
                diary off;
                movefile(['Duplicate' num2str(runs  - 1 ) '.mdl'],[pwd '\DataType'],'f');
                movefile(['Duplicate' num2str(runs  - 1 ) '_changes.txt'],[pwd '\DataType'],'f');
            else
                close_system(['Duplicate' num2str(runs)],0);
            end
        end
        
    end
    
    if strcmp(blks{m},'RelationalOperator')
        for k = 1:size(relation,2)
            rep = relation{k};
            open_system('Duplicate');
            save_system('Duplicate',['Duplicate' num2str(runs)]);
            open_system(['Duplicate' num2str(runs)]);
            tmp = regexprep(model{m},'Original',['Duplicate' num2str(runs)],'once');
            val = get_param(tmp,'Operator');
            if ~strcmp(val,rep)
                set_param(tmp,'Operator',rep);
                set_param(tmp,'BackgroundColor','red');
                runs = runs + 1;
                save_system(['Duplicate' num2str(runs  - 1 )]);
                %close_system(['Duplicate' num2str(runs  - 1 )],0);
                CreateHarness;
                open_system(modelname);
                eval(['diary Duplicate' num2str(runs - 1) '_changes.txt']);
                disp(['Changed RelationalOperator in ' model{m} ' Relation from ' get_param(model{m},'Operator') ' to ' rep]);
                diary off;
                movefile(['Duplicate' num2str(runs  - 1 ) '.mdl'],[pwd '\Relational'],'f');
                movefile(['Duplicate' num2str(runs  - 1 ) '_changes.txt'],[pwd '\Relational'],'f');
            else
                close_system(['Duplicate' num2str(runs)],0);
            end
        end
        
    end
    
    % If a constant block is encountered change the value by 0.1 percent
    if strcmp(blks{m},'Constant')
        r = 1;
        for k = 1:2
            load_system(modelname);
            tmp1 = str2num(get_param(model{m},'Value'));
            tmp2=0.0; %#ok<NASGU>
            if isempty(tmp1)
                tmp2 = rand * 200;
            elseif tmp1 == 0
                tmp2 = rand * 200;
            else
                tmp2 = tmp1;
            end
            if r
                rep = tmp2 + (tmp2*0.001);
            else
                rep = tmp2 - (tmp2*0.001);
            end
            open_system('Duplicate');
            save_system('Duplicate',['Duplicate' num2str(runs)]);
            open_system(['Duplicate' num2str(runs)]);
            tmp = regexprep(model{m},'Original',['Duplicate' num2str(runs)],'once');
            set_param(tmp,'Value',num2str(rep));
            set_param(tmp,'BackgroundColor','red');
            runs = runs + 1;
            save_system(['Duplicate' num2str(runs  - 1 )]);
            %close_system(['Duplicate' num2str(runs  - 1 )],0);
            CreateHarness;
            open_system(modelname);
            eval(['diary Duplicate' num2str(runs - 1) '_changes.txt']);
            disp(['Changed Constant in ' model{m} ' Relation from ' num2str(tmp1) ' to ' num2str(rep)]);
            diary off;
            movefile(['Duplicate' num2str(runs  - 1 ) '.mdl'],[pwd '\DataType'],'f');
            movefile(['Duplicate' num2str(runs  - 1 ) '_changes.txt'],[pwd '\DataType'],'f');
            r = 0;
        end
        
    end
    
    if strcmp(blks{m},'DiscreteFilter')
        p = true;
        for l = 1:2
            r = true;
            for k = 1:2
                num = 0;
                den = 0;
                load_system(modelname);
                if r
                    tmp1 = get_param(model{m},'Numerator');
                    num = 1;
                else
                    tmp1 = get_param(model{m},'Denominator');
                    den = 1;
                end
                if ischar(tmp1)
                    tmp1 = str2num(tmp1); %#ok<*ST2NM>
                end
                if p
                    rep = ['[' num2str(tmp1 + (rand * 0.1)) ']'];
                else
                    rep = ['[' num2str(tmp1 - (rand * 0.1)) ']' ];
                end
                open_system('Duplicate');
                save_system('Duplicate',['Duplicate' num2str(runs)]);
                open_system(['Duplicate' num2str(runs)]);
                
                tmp = regexprep(model{m},'Original',['Duplicate' num2str(runs)],'once');
                
                
                if num
                    set_param(tmp,'Numerator',rep);
                    set_param(tmp,'BackgroundColor','red');
                    runs = runs + 1;
                    save_system(['Duplicate' num2str(runs  - 1 )]);
                    %close_system(['Duplicate' num2str(runs  - 1 )],0);
                    CreateHarness;
                    open_system(modelname);
                    eval(['diary Duplicate' num2str(runs - 1) '_changes.txt']);
                    disp(['Changed Numerator in ' model{m} ' Relation from ' num2str(tmp1) ' to ' rep]);
                    diary off;
                    movefile(['Duplicate' num2str(runs  - 1 ) '.mdl'],[pwd '\Filter'],'f');
                    movefile(['Duplicate' num2str(runs  - 1 ) '_changes.txt'],[pwd '\Filter'],'f');
                    
                end
                if den
                    set_param(tmp,'Denominator',rep);
                    set_param(tmp,'BackgroundColor','red');
                    runs = runs + 1;
                    save_system(['Duplicate' num2str(runs  - 1 )]);
                    %close_system(['Duplicate' num2str(runs  - 1 )],0);
                    CreateHarness;
                    open_system(modelname);
                    eval(['diary Duplicate' num2str(runs - 1) '_changes.txt']);
                    disp(['Changed Denominator in ' model{m} ' Relation from ' num2str(tmp1) ' to ' rep]);
                    diary off;
                    movefile(['Duplicate' num2str(runs  - 1 ) '.mdl'],[pwd '\Filter'],'f');
                    movefile(['Duplicate' num2str(runs  - 1 ) '_changes.txt'],[pwd '\Filter'],'f');
                end
                r = ~r;
            end
            p = ~p;
        end
        
    end
    
    if strcmp(blks{m},'DiscreteIntegrator')
        
        for k = 1:2
            for l = 1:5
                
                load_system(modelname);
                tmp1 = str2num(get_param(model{m},'InitialCondition'));
                tmp2 = str2num(get_param(model{m},'gainval'));
                uptmp = str2num(get_param(model{m},'UpperSaturationLimit'));
                lowtmp = str2num(get_param(model{m},'LowerSaturationLimit'));
                lim = get_param(model{m},'LimitOutput');
                
                if tmp1 == 0
                    tmp1 = rand*5+0.01;
                end
                if tmp2 == 0
                    tmp2 = rand*5+0.01;
                end
                
                rep1 = num2str(tmp1 + (tmp1 * 0.001));
                rep2 = num2str(tmp2 + (tmp2 * 0.001));
                rep3 = num2str(uptmp + (uptmp * 0.001));
                rep4 = num2str(lowtmp + (lowtmp * 0.001));
                
                
                open_system('Duplicate');
                save_system('Duplicate',['Duplicate' num2str(runs)]);
                open_system(['Duplicate' num2str(runs)]);
                
                tmp = regexprep(model{m},'Original',['Duplicate' num2str(runs)],'once');
                
                switch l
                    case 1
                        set_param(tmp,'InitialCondition',rep1);
                        set_param(tmp,'BackgroundColor','red');
                        runs = runs + 1;
                        save_system(['Duplicate' num2str(runs  - 1 )]);
                        %close_system(['Duplicate' num2str(runs  - 1 )],0);
                        CreateHarness;
                        open_system(modelname);
                        eval(['diary Duplicate' num2str(runs - 1) '_changes.txt']);
                        disp(['Changed Integrator initial condition in ' model{m} ' Relation from ' num2str(tmp1) ' to ' rep1]);
                        diary off;
                        movefile(['Duplicate' num2str(runs  - 1 ) '.mdl'],[pwd '\Integrator'],'f');
                        movefile(['Duplicate' num2str(runs  - 1 ) '_changes.txt'],[pwd '\Integrator'],'f');
                        
                    case 2
                        set_param(tmp,'gainval',rep2);
                        set_param(tmp,'BackgroundColor','red');
                        runs = runs + 1;
                        save_system(['Duplicate' num2str(runs  - 1 )]);
                        %close_system(['Duplicate' num2str(runs  - 1 )],0);
                        CreateHarness;
                        open_system(modelname);
                        eval(['diary Duplicate' num2str(runs - 1) '_changes.txt']);
                        disp(['Changed Integrator gain in ' model{m} ' Relation from ' num2str(tmp2) ' to ' rep2]);
                        diary off;
                        movefile(['Duplicate' num2str(runs  - 1 ) '.mdl'],[pwd '\Integrator'],'f');
                        movefile(['Duplicate' num2str(runs  - 1 ) '_changes.txt'],[pwd '\Integrator'],'f');
                        
                    case 3
                        set_param(tmp,'LimitOutput','On');
                        set_param(tmp,'UpperSaturationLimit',rep3);
                        set_param(tmp,'BackgroundColor','red');
                        runs = runs + 1;
                        save_system(['Duplicate' num2str(runs  - 1 )]);
                        %close_system(['Duplicate' num2str(runs  - 1 )],0);
                        CreateHarness;
                        open_system(modelname);
                        eval(['diary Duplicate' num2str(runs - 1) '_changes.txt']);
                        disp(['Changed Integrator upper limit in ' model{m} ' Relation from ' num2str(uptmp) ' to ' rep3]);
                        diary off;
                        movefile(['Duplicate' num2str(runs  - 1 ) '.mdl'],[pwd '\Integrator'],'f');
                        movefile(['Duplicate' num2str(runs  - 1 ) '_changes.txt'],[pwd '\Integrator'],'f');
                    case 4
                        set_param(tmp,'LimitOutput','On');
                        set_param(tmp,'LowerSaturationLimit',rep4);
                        set_param(tmp,'BackgroundColor','red');
                        runs = runs + 1;
                        save_system(['Duplicate' num2str(runs  - 1 )]);
                        %close_system(['Duplicate' num2str(runs  - 1 )],0);
                        CreateHarness;
                        open_system(modelname);
                        eval(['diary Duplicate' num2str(runs - 1) '_changes.txt']);
                        disp(['Changed Integrator lower limit in ' model{m} ' Relation from ' num2str(lowtmp) ' to ' rep4]);
                        diary off;
                        movefile(['Duplicate' num2str(runs  - 1 ) '.mdl'],[pwd '\Integrator'],'f');
                        movefile(['Duplicate' num2str(runs  - 1 ) '_changes.txt'],[pwd '\Integrator'],'f');
                        
                    case 5
                        if strcmp(lim,'On')
                            set_param(tmp,'LimitOutput','Off');
                            set_param(tmp,'BackgroundColor','red');
                            runs = runs + 1;
                            save_system(['Duplicate' num2str(runs  - 1 )]);
                            %close_system(['Duplicate' num2str(runs  - 1 )],0);
                            CreateHarness;
                            open_system(modelname);
                            eval(['diary Duplicate' num2str(runs - 1) '_changes.txt']);
                            disp(['Changed Integrator Limit option off in ' model{m} ' Relation from ' num2str(lowtmp) ' to ' rep4]);
                            diary off;
                            movefile(['Duplicate' num2str(runs  - 1 ) '.mdl'],[pwd '\Integrator'],'f');
                            movefile(['Duplicate' num2str(runs  - 1 ) '_changes.txt'],[pwd '\Integrator'],'f');
                        else
                            set_param(tmp,'LimitOutput','On');
                            set_param(tmp,'UpperSaturationLimit',num2str(rand * 10000));
                            set_param(tmp,'LowerSaturationLimit',num2str(rand * -10000));
                            set_param(tmp,'BackgroundColor','red');
                            runs = runs + 1;
                            save_system(['Duplicate' num2str(runs  - 1 )]);
                            %close_system(['Duplicate' num2str(runs  - 1 )],0);
                            CreateHarness;
                            open_system(modelname);
                            eval(['diary Duplicate' num2str(runs - 1) '_changes.txt']);
                            disp(['Changed Integrator Limit option on in ' model{m} ' Relation from ' num2str(lowtmp) ' to ' rep4]);
                            diary off;
                            movefile(['Duplicate' num2str(runs  - 1 ) '.mdl'],[pwd '\Integrator'],'f');
                            movefile(['Duplicate' num2str(runs  - 1 ) '_changes.txt'],[pwd '\Integrator'],'f');
                        end
                end
            end
        end
        
    end
    
    if strcmp(blks{m},'Switch')
        f = true;
        for l = 1:2
            if f
                for k = 1:size(swit,2)
                    rep = swit{k};
                    open_system('Duplicate');
                    save_system('Duplicate',['Duplicate' num2str(runs)]);
                    open_system(['Duplicate' num2str(runs)]);
                    tmp = regexprep(model{m},'Original',['Duplicate' num2str(runs)],'once');
                    val = get_param(tmp,'Criteria');
                    if ~strcmp(val,rep)
                        set_param(tmp,'Criteria',rep);
                        set_param(tmp,'BackgroundColor','red');
                        runs = runs + 1;
                        save_system(['Duplicate' num2str(runs  - 1 )]);
                        %close_system(['Duplicate' num2str(runs  - 1 )],0);
                        CreateHarness;
                        open_system(modelname);
                        eval(['diary Duplicate' num2str(runs - 1) '_changes.txt']);
                        disp(['Changed Switching condition in ' model{m} ' Relation from ' get_param(model{m},'Criteria') ' to ' rep]);
                        diary off;
                        movefile(['Duplicate' num2str(runs  - 1 ) '.mdl'],[pwd '\Switch'],'f');
                        movefile(['Duplicate' num2str(runs  - 1 ) '_changes.txt'],[pwd '\Switch'],'f');
                    else
                        close_system(['Duplicate' num2str(runs)],0);
                    end
                end
                f = ~f;
            else
                
                open_system('Duplicate');
                save_system('Duplicate',['Duplicate' num2str(runs)]);
                open_system(['Duplicate' num2str(runs)]);
                tmp = regexprep(model{m},'Original',['Duplicate' num2str(runs)],'once');
                val = get_param(tmp,'Criteria');
                if ~strcmp(val,'u2 ~= 0')
                    rep = num2str(str2num(get_param(tmp,'Threshold')) + rand);
                    set_param(tmp,'Threshold',rep);
                    set_param(tmp,'BackgroundColor','red');
                    runs = runs + 1;
                    save_system(['Duplicate' num2str(runs  - 1 )]);
                    CreateHarness;
                    open_system(modelname);
                    eval(['diary Duplicate' num2str(runs - 1) '_changes.txt']);
                    disp(['Changed Switching Threshold in ' model{m} ' Relation from ' get_param(model{m},'Threshold') ' to ' rep]);
                    diary off;
                    movefile(['Duplicate' num2str(runs  - 1 ) '.mdl'],[pwd '\Switch'],'f');
                    movefile(['Duplicate' num2str(runs  - 1 ) '_changes.txt'],[pwd '\Switch'],'f');
                else
                    close_system(['Duplicate' num2str(runs)],0);
                end
            end
            
        end
    end
    
    if strcmp(blks{m},'Saturate')
        r = 1;
        for k = 1:2
            load_system(modelname);
            tmp1 = str2num(get_param(model{m},'UpperLimit'));
            tmp2 = str2num(get_param(model{m},'LowerLimit'));
            if r
                rep1 = tmp1 + (tmp1 * 0.001);
                rep2 = tmp2 + (tmp2 * 0.001);
            else
                rep1 = tmp1 - (tmp1 * 0.001);
                rep2 = tmp2 - (tmp2 * 0.001);
            end
            open_system('Duplicate');
            save_system('Duplicate',['Duplicate' num2str(runs)]);
            open_system(['Duplicate' num2str(runs)]);
            tmp = regexprep(model{m},'Original',['Duplicate' num2str(runs)],'once');
            set_param(tmp,'UpperLimit',num2str(rep1));
            set_param(tmp,'LowerLimit',num2str(rep2));
            set_param(tmp,'BackgroundColor','red');
            runs = runs + 1;
            save_system(['Duplicate' num2str(runs  - 1 )]);
            %close_system(['Duplicate' num2str(runs  - 1 )],0);
            CreateHarness;
            open_system(modelname);
            eval(['diary Duplicate' num2str(runs - 1) '_changes.txt']);
            disp(['Changed upper limit in ' model{m} ' Relation from ' num2str(tmp1) ' to ' num2str(rep1)]);
            disp(['Changed lower limit in ' model{m} ' Relation from ' num2str(tmp2) ' to ' num2str(rep2)]);
            diary off;
            movefile(['Duplicate' num2str(runs  - 1 ) '.mdl'],[pwd '\Saturation'],'f');
            movefile(['Duplicate' num2str(runs  - 1 ) '_changes.txt'],[pwd '\Saturation'],'f');
            r = 0;
        end
        
    end
    
    if strcmp(blks{m},'Lookup')
        f = true;
        for l = 1:2
            
            if f
                j = true;
                for k = 1:2
                    
                    open_system('Duplicate');
                    save_system('Duplicate',['Duplicate' num2str(runs)]);
                    open_system(['Duplicate' num2str(runs)]);
                    tmp = regexprep(model{m},'Original',['Duplicate' num2str(runs)],'once');
                    val1 = str2num(get_param(tmp,'InputValues'));
                    val2 = str2num(get_param(tmp,'Table'));
                    
                    if j
                        c = randi(length(val1));
                        val1(c) = val1(c) * rand;
                        val1 = sort(val1);
                        rep = ['[' num2str(val1) ']'];
                        set_param(tmp,'InputValues',rep);
                        set_param(tmp,'BackgroundColor','red');
                        runs = runs + 1;
                        save_system(['Duplicate' num2str(runs  - 1 )]);
                        %close_system(['Duplicate' num2str(runs  - 1 )],0);
                        CreateHarness;
                        open_system(modelname);
                        eval(['diary Duplicate' num2str(runs - 1) '_changes.txt']);
                        disp(['Changed Input array in ' model{m} ' Relation from ' get_param(model{m},'InputValues') ' to ' rep]);
                        diary off;
                        movefile(['Duplicate' num2str(runs  - 1 ) '.mdl'],[pwd '\Lookup'],'f');
                        movefile(['Duplicate' num2str(runs  - 1 ) '_changes.txt'],[pwd '\Lookup'],'f');
                        j = ~j;
                    else
                        c = randi(length(val2));
                        val2(c) = val2(c) * rand;
                        rep = ['[' num2str(val2) ']'];
                        set_param(tmp,'Table',rep);
                        set_param(tmp,'BackgroundColor','red');
                        runs = runs + 1;
                        save_system(['Duplicate' num2str(runs  - 1 )]);
                        %close_system(['Duplicate' num2str(runs  - 1 )],0);
                        CreateHarness;
                        open_system(modelname);
                        eval(['diary Duplicate' num2str(runs - 1) '_changes.txt']);
                        disp(['Changed Table data in ' model{m} ' Relation from ' get_param(model{m},'Table') ' to ' rep]);
                        diary off;
                        movefile(['Duplicate' num2str(runs  - 1 ) '.mdl'],[pwd '\Lookup'],'f');
                        movefile(['Duplicate' num2str(runs  - 1 ) '_changes.txt'],[pwd '\Lookup'],'f');
                    end
                end
                f = ~f;
            else
                for k = 1:size(look,2)
                    rep = look{k};
                    open_system('Duplicate');
                    save_system('Duplicate',['Duplicate' num2str(runs)]);
                    open_system(['Duplicate' num2str(runs)]);
                    tmp = regexprep(model{m},'Original',['Duplicate' num2str(runs)],'once');
                    val = get_param(tmp,'LookUpMeth');
                    if ~strcmp(val,rep)
                        set_param(tmp,'LookUpMeth',rep);
                        set_param(tmp,'BackgroundColor','red');
                        runs = runs + 1;
                        save_system(['Duplicate' num2str(runs  - 1 )]);
                        %close_system(['Duplicate' num2str(runs  - 1 )],0);
                        CreateHarness;
                        open_system(modelname);
                        eval(['diary Duplicate' num2str(runs - 1) '_changes.txt']);
                        disp(['Changed Look up method in ' model{m} ' Relation from ' get_param(model{m},'LookUpMeth') ' to ' rep]);
                        diary off;
                        movefile(['Duplicate' num2str(runs  - 1 ) '.mdl'],[pwd '\Lookup'],'f');
                        movefile(['Duplicate' num2str(runs  - 1 ) '_changes.txt'],[pwd '\Lookup'],'f');
                    else
                        close_system(['Duplicate' num2str(runs)],0);
                    end
                end
            end
            
        end
    end
    
    if strcmp(blks{m},'Backlash')
        r = 0;
        for k = 1:2
            load_system(modelname);
            tmp1 = str2num(get_param(model{m},'BacklashWidth'));
            if isempty(tmp1)
                tmp1 = rand * 200;
            end
            if r
                rep = tmp1 + (tmp1 * 0.001);
            else
                rep = tmp1 - (tmp1 * 0.001);
            end
            open_system('Duplicate');
            save_system('Duplicate',['Duplicate' num2str(runs)]);
            open_system(['Duplicate' num2str(runs)]);
            tmp = regexprep(model{m},'Original',['Duplicate' num2str(runs)],'once');
            set_param(tmp,'BacklashWidth',num2str(rep));
            set_param(tmp,'BackgroundColor','red');
            runs = runs + 1;
            save_system(['Duplicate' num2str(runs  - 1 )]);
            %close_system(['Duplicate' num2str(runs  - 1 )],0);
            CreateHarness;
            open_system(modelname);
            eval(['diary Duplicate' num2str(runs - 1) '_changes.txt']);
            disp(['Changed BackLash width ' model{m} ' Relation from ' num2str(tmp1) ' to ' num2str(rep)]);
            diary off;
            movefile(['Duplicate' num2str(runs  - 1 ) '.mdl'],[pwd '\Backlash'],'f');
            movefile(['Duplicate' num2str(runs  - 1 ) '_changes.txt'],[pwd '\Backlash'],'f');
            r = r + 1;
        end
        
    end
    
    if strcmp(blks{m},'S-Function')
        r = 0;
        try
            for k = 1:2
                load_system(modelname);
                tmp1 = str2num(get_param(model{m},'vinit'));
                if r
                    rep = tmp1 + 1;
                else
                    rep = tmp1 - 1;
                end
                open_system('Duplicate');
                save_system('Duplicate',['Duplicate' num2str(runs)]);
                open_system(['Duplicate' num2str(runs)]);
                tmp = regexprep(model{m},'Original',['Duplicate' num2str(runs)],'once');
                set_param(tmp,'vinit',num2str(rep));
                set_param(tmp,'BackgroundColor','red');
                runs = runs + 1;
                save_system(['Duplicate' num2str(runs  - 1 )]);
                %close_system(['Duplicate' num2str(runs  - 1 )],0);
                CreateHarness;
                open_system(modelname);
                eval(['diary Duplicate' num2str(runs - 1) '_changes.txt']);
                disp(['Changed Delay Initial condition ' model{m} ' Relation from ' num2str(tmp1) ' to ' num2str(rep)]);
                diary off;
                movefile(['Duplicate' num2str(runs  - 1 ) '.mdl'],[pwd '\Delay'],'f');
                movefile(['Duplicate' num2str(runs  - 1 ) '_changes.txt'],[pwd '\Delay'],'f');
                r = r + 1;
            end
        catch none
        end
        r = 0;
        try
            for k = 1:2
                load_system(modelname);
                tmp1 = str2num(get_param(model{m},'NumDelays'));
                if r
                    rep = tmp1 + 1;
                else
                    rep = tmp1 - 1;
                    if tmp1 == 1
                        rep = tmp1 + 2;
                    end
                end
                open_system('Duplicate');
                save_system('Duplicate',['Duplicate' num2str(runs)]);
                open_system(['Duplicate' num2str(runs)]);
                tmp = regexprep(model{m},'Original',['Duplicate' num2str(runs)],'once');
                set_param(tmp,'NumDelays',num2str(rep));
                set_param(tmp,'BackgroundColor','red');
                runs = runs + 1;
                save_system(['Duplicate' num2str(runs  - 1 )]);
                %close_system(['Duplicate' num2str(runs  - 1 )],0);
                CreateHarness;
                open_system(modelname);
                eval(['diary Duplicate' num2str(runs - 1) '_changes.txt']);
                disp(['Changed Number of Delays ' model{m} ' Relation from ' num2str(tmp1) ' to ' num2str(rep)]);
                diary off;
                movefile(['Duplicate' num2str(runs  - 1 ) '.mdl'],[pwd '\Delay'],'f');
                movefile(['Duplicate' num2str(runs  - 1 ) '_changes.txt'],[pwd '\Delay'],'f');
                r = r + 1;
            end
        catch none
        end
    end
    
    
    
    load_system('Simulink');
    if strcmp(blks{m},'S-Function') || strcmp(blks{m},'Backlash') || strcmp(blks{m},'Lookup') || strcmp(blks{m},'Saturate') || strcmp(blks{m},'Switch') || strcmp(blks{m},'DiscreteIntegrator') || strcmp(blks{m},'DiscreteFilter') || strcmp(blks{m},'Constant') || strcmp(blks{m},'RelationalOperator') || strcmp(blks{m},'DataTypeConversion') || strcmp(blks{m},'Sum') || strcmp(blks{m},'Logic') || strcmp(blks{m},'Outport')
        lin = get_param(model{m},'LineHandles');
        for k = 1:size(lin.Inport,2)
            r = 1;
            for j = 1:2
                open_system('Duplicate');
                save_system('Duplicate',['Duplicate' num2str(runs)]);
                open_system(['Duplicate' num2str(runs)]);
                nam = regexprep(model{m},modelname,['Duplicate' num2str(runs)]);
                line = get_param(nam,'LineHandles');
                a = get_param(nam,'PortConnectivity');
                nam1 = get_param(a(k).SrcBlock,'Name');
                tempname = regexprep(nam,['/' get_param(nam,'Name')],'','once');
                
                delete_line(line.Inport(k));
                set_param(nam,'BackgroundColor','red');
                runs = runs + 1;
                tmp1 = rand * 0.1 * r;
                b = add_block('Simulink/Math Operations/Gain',[tempname '/AddedGain'],'Gain',num2str(tmp1));
                add_line(tempname,'AddedGain/1',[get_param(nam,'Name') '/' num2str(k)],'autorouting','on');
                add_line(tempname,[nam1 '/1'],'AddedGain/1','autorouting','on');
                save_system(['Duplicate' num2str(runs  - 1 )]);
                CreateHarness;
                eval(['diary Duplicate' num2str(runs - 1) '_changes.txt']);
                disp(['Added gain of ' num2str(tmp1) ' to input connected to port ' num2str(k) ' in block ' model{m}]);
                diary off;
                movefile(['Duplicate' num2str(runs  - 1 ) '.mdl'],[pwd '\Lines'],'f');
                movefile(['Duplicate' num2str(runs  - 1 ) '_changes.txt'],[pwd '\Lines'],'f');
                close_system(['Duplicate' num2str(runs  - 1 )],0);
                r = 0;
            end
        end
        
    end
    
end
close_system(modelname,0);

